package com.zanxiang.game.module.manage.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.zanxiang.erp.security.annotation.PreAuthorize;
import com.zanxiang.game.module.manage.pojo.params.*;
import com.zanxiang.game.module.manage.pojo.vo.RoleAssignRecordListVO;
import com.zanxiang.game.module.manage.service.IRoleAssignRecordService;
import com.zanxiang.module.util.pojo.ResultVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@Api(tags = {"角色指派记录"})
@RestController
@RequestMapping("/role/assign/record")
@Slf4j
public class RoleAssignController {

    @Autowired
    private IRoleAssignRecordService roleAssignRecordService;

    @ApiOperation(value = "指派人员")
    @PostMapping(value = "/config/sysUser")
    @PreAuthorize(permissionKey = "manage:roleAssign:configSysUser")
    public ResultVO<Boolean> configSysUser(@Validated @RequestBody RoleSysUserAssignParam param) {
        return ResultVO.ok(roleAssignRecordService.configSysUser(param));
    }

    @ApiOperation(value = "指派记录修改")
    @PostMapping(value = "/modify")
    @PreAuthorize(permissionKey = "manage:roleAssign:modify")
    public ResultVO<Boolean> modify(@Validated @RequestBody RoleAssignRecordAddParam param) {
        return ResultVO.ok(roleAssignRecordService.modify(param));
    }

    @ApiOperation(value = "指派记录列表查询")
    @PostMapping(value = "/list")
    @PreAuthorize(permissionKey = "manage:roleAssign:list")
    public ResultVO<IPage<RoleAssignRecordListVO>> list(@Validated @RequestBody RoleAssignRecordListParam param) {
        return ResultVO.ok(roleAssignRecordService.listOfPage(param));
    }

    @ApiOperation(value = "指派记录删除")
    @DeleteMapping(value = "/delete/{id}")
    @PreAuthorize(permissionKey = "manage:roleAssign:delete")
    public ResultVO<Boolean> deleteById(@PathVariable Long id) {
        return ResultVO.ok(roleAssignRecordService.deleteById(id));
    }
}
