package com.zanxiang.game.module.manage.pojo.dto;

import com.zanxiang.erp.security.util.SecurityUtil;
import com.zanxiang.game.module.manage.enums.KfRoomMsgTypeEnum;
import com.zanxiang.game.module.manage.enums.KfWebSocketMsgEnum;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.time.LocalDateTime;
import java.util.List;

/**
 * @author : lingfeng
 * @time : 2024-02-26
 * @description : WebSocket客服消息通信返回
 */
@Data
@Builder
@AllArgsConstructor
@NoArgsConstructor
public class KfWebSocketMsgDTO {

    private static final long serialVersionUID = 1L;

    /**
     * webSocket消息类型
     */
    private KfWebSocketMsgEnum webSocketMsgType;

    /**
     * 客服id, 存在 : 表示该消息发送给指定客服, 不存在 : 表示该消息发送给所有在线客服
     */
    private Long kfUserId;

    /**
     * 分页信息
     */
    private PageBean page;

    /**
     * 游戏id
     */
    private Long gameId;

    /**
     * 房间id
     */
    private Long roomId;

    /**
     * 结果信息
     */
    private ResultBean result;

    /**
     * 游戏列表, 全量不分页
     */
    private List<GameBean> gameList;

    /**
     * 待接入列表, 全量不分页
     */
    private List<WaitUserBean> waitUserList;

    /**
     * 房间列表, 已接入房间全量返回, 已经结束房间分页返回, 配套返回 page 信息
     */
    private List<RoomBean> roomList;

    /**
     * 房间消息 : 房间消息推送和历史消息列表都用该字段, 历史消息列表会配套返回 page 信息
     */
    private List<RoomMsgBean> roomMsgList;

    public KfWebSocketMsgDTO(KfWebSocketMsgEnum webSocketMsgType, Integer errorCode, String errorMsg) {
        this.webSocketMsgType = webSocketMsgType;
        this.kfUserId = SecurityUtil.getUserId();
        this.result = ResultBean.builder()
                .errorCode(errorCode)
                .errorMsg(errorMsg)
                .build();
    }

    public static KfWebSocketMsgDTO ok(KfWebSocketMsgEnum webSocketMsgType) {
        return new KfWebSocketMsgDTO(webSocketMsgType, 0, "success");
    }

    public static KfWebSocketMsgDTO fail(KfWebSocketMsgEnum webSocketMsgType, String errorMsg) {
        return new KfWebSocketMsgDTO(webSocketMsgType, 400, errorMsg);
    }

    @Data
    @Builder
    @AllArgsConstructor
    @NoArgsConstructor
    public static class UserBean {

        /**
         * 玩家id
         */
        private Long userId;

        /**
         * 最近角色id
         */
        private String lastRoleId;

        /**
         * 最近角色名称
         */
        private String lastRoleName;

        /**
         * 角色服务器id
         */
        private String serverId;

        /**
         * 角色服务器名称
         */
        private String serverName;

        /**
         * 充值笔数
         */
        private Long orderCount;

        /**
         * 最大充值金额
         */
        private Long orderMaxAmount;

        /**
         * 累计充值金额
         */
        private Long orderAmountSum;
    }

    @Data
    @Builder
    @AllArgsConstructor
    @NoArgsConstructor
    public static class GameRoleBean {

        /**
         * 玩家id
         */
        private Long userId;

        /**
         * 最近角色id
         */
        private String lastRoleId;

        /**
         * 最近角色名称
         */
        private String lastRoleName;

        /**
         * 角色服务器id
         */
        private String serverId;

        /**
         * 角色服务器名称
         */
        private String serverName;

        /**
         * 充值笔数
         */
        private Long orderCount;

        /**
         * 最大充值金额
         */
        private Long orderMaxAmount;

        /**
         * 累计充值金额
         */
        private Long orderAmountSum;
    }

    @Data
    @Builder
    @AllArgsConstructor
    @NoArgsConstructor
    public static class OrderBean {

        /**
         * 玩家id
         */
        private Long userId;

        /**
         * 最近角色id
         */
        private String lastRoleId;

        /**
         * 最近角色名称
         */
        private String lastRoleName;

        /**
         * 角色服务器id
         */
        private String serverId;

        /**
         * 角色服务器名称
         */
        private String serverName;

        /**
         * 充值笔数
         */
        private Long orderCount;

        /**
         * 最大充值金额
         */
        private Long orderMaxAmount;

        /**
         * 累计充值金额
         */
        private Long orderAmountSum;
    }

    @Data
    @Builder
    @AllArgsConstructor
    @NoArgsConstructor
    public static class GameBean {

        private static final long serialVersionUID = 1L;

        /**
         * 游戏id
         */
        private Long gameId;

        /**
         * 小程序名称
         */
        private String appName;

        /**
         * 未读消息总数
         */
        private Integer unReadMsgCount;
    }

    @Data
    @Builder
    @AllArgsConstructor
    @NoArgsConstructor
    public static class WaitUserBean {

        private static final long serialVersionUID = 1L;

        /**
         * 玩家openId
         */
        private String openId;

        /**
         * 消息游戏id
         */
        private Long gameId;

        /**
         * 最近角色id
         */
        private String lastRoleId;

        /**
         * 最近角色名称
         */
        private String lastRoleName;

        /**
         * 未读消息列表
         */
        private List<WaitUserMsgBean> waitUserMsgList;

        /**
         * 未读消息总数
         */
        private Long unReadMsgCount;

        /**
         * 开始等待时间
         */
        private LocalDateTime waitStartTime;
    }

    @Data
    @Builder
    @AllArgsConstructor
    @NoArgsConstructor
    public static class WaitUserMsgBean {

        /**
         * 消息id
         */
        private String msgId;

        /**
         * 消息类型
         */
        private String msgType;

        /**
         * 创建时间
         */
        private LocalDateTime createTime;

        /**
         * 消息内容
         */
        private String content;

    }

    @Data
    @Builder
    @AllArgsConstructor
    @NoArgsConstructor
    public static class RoomBean {

        private static final long serialVersionUID = 1L;

        /**
         * 房间id
         */
        private Long roomId;

        /**
         * 消息游戏id
         */
        private Long gameId;

        /**
         * 玩家open_id
         */
        private String openId;

        /**
         * 最近角色id
         */
        private String lastRoleId;

        /**
         * 最近角色名称
         */
        private String lastRoleName;

        /**
         * 客服id
         */
        private Long kfUserId;

        /**
         * 房间在线状态
         */
        private Boolean online;

        /**
         * 最后一条消息
         */
        private String lastMsg;

        /**
         * 最后一条消息时间
         */
        private LocalDateTime lastMsgTime;

        /**
         * 未读消息总数
         */
        private Integer unReadMsgCount;
    }

    @Data
    @Builder
    @AllArgsConstructor
    @NoArgsConstructor
    public static class RoomMsgBean {

        private static final long serialVersionUID = 1L;

        /**
         * 消息id
         */
        private String msgId;

        /**
         * 消息类型
         */
        private KfRoomMsgTypeEnum msgType;

        /**
         * 消息游戏id
         */
        private Long gameId;

        /**
         * 房间id
         */
        private Long roomId;

        /**
         * 已读状态
         */
        private Boolean readStatus;

        /**
         * 消息归属
         */
        private String msgOwner;

        /**
         * 消息内容
         */
        private MsgContentBean content;

        /**
         * 创建时间
         */
        private LocalDateTime createTime;
    }

    @Data
    @Builder
    @AllArgsConstructor
    @NoArgsConstructor
    public static class MsgContentBean {

        private static final long serialVersionUID = 1L;

        /**
         * 文本内容
         */
        private String text;

        /**
         * 图片地址
         */
        private String image;

        /**
         * 图文消息
         */
        private LinkBean link;
    }

    @Data
    @Builder
    @AllArgsConstructor
    @NoArgsConstructor
    public static class LinkBean {

        private static final long serialVersionUID = 1L;

        /**
         * 标题
         */
        private String title;

        /**
         * 描述
         */
        private String description;

        /**
         * 跳转链接
         */
        private String url;

        /**
         * 图片地址
         */
        private String thumbUrl;
    }

    @Data
    @Builder
    @AllArgsConstructor
    @NoArgsConstructor
    public static class PageBean {

        private static final long serialVersionUID = 1L;

        /**
         * 当前页数
         */
        private Long pageNum;

        /**
         * 单页数量
         */
        private Long pageSize;

        /**
         * 总页数
         */
        private Long pageTotal;

        /**
         * 数据总量
         */
        private Long total;
    }

    @Data
    @Builder
    @AllArgsConstructor
    @NoArgsConstructor
    public static class ResultBean {

        private static final long serialVersionUID = 1L;

        /**
         * 0 : 成功, 其他失败
         */
        private Integer errorCode;

        /**
         * 错误消息
         */
        private String errorMsg;
    }

}
