package com.zanxiang.game.module.manage.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.zanxiang.game.module.manage.constant.RedisKeyConstant;
import com.zanxiang.game.module.manage.enums.KfRoomMsgOwnerEnum;
import com.zanxiang.game.module.manage.enums.KfRoomMsgTypeEnum;
import com.zanxiang.game.module.manage.enums.KfWebSocketMsgEnum;
import com.zanxiang.game.module.manage.pojo.dto.KfAppletMsgDTO;
import com.zanxiang.game.module.manage.pojo.dto.KfWebSocketMsgDTO;
import com.zanxiang.game.module.manage.service.*;
import com.zanxiang.game.module.mybatis.entity.*;
import com.zanxiang.module.util.JsonUtil;
import com.zanxiang.module.util.bean.BeanUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.Collections;
import java.util.List;
import java.util.Objects;

/**
 * @author : lingfeng
 * @time : 2024-02-28
 * @description : 小程序客服消息处理
 */
@Slf4j
@Service
public class KfAppletMsgServiceImpl implements IKfAppletMsgService {

    @Autowired
    private RedisTemplate<String, String> redisTemplate;

    @Autowired
    private IKfSessionUserService kfSessionUserService;

    @Autowired
    private IKfRoomService kfRoomService;

    @Autowired
    private IGameAppletService gameAppletService;

    @Autowired
    private IUserService userService;

    @Autowired
    private IGameUserRoleService gameUserRoleService;

    @Autowired
    private IKfRoomMsgService kfRoomMsgService;

    @Override
    public void appletMsg(String postData) {
        log.error("接收到SDK转发的小程序消息, postData : {}", postData);
        KfAppletMsgDTO kfAppletMsgDTO = JsonUtil.toObj(postData, KfAppletMsgDTO.class);
        GameApplet gameApplet = gameAppletService.getOne(new LambdaQueryWrapper<GameApplet>()
                .eq(GameApplet::getGhId, kfAppletMsgDTO.getToUserName()));
        //小游戏信息不存在, 消息不处理
        if (gameApplet == null) {
            return;
        }
        //用户进入会话事件
        if (Objects.equals(kfAppletMsgDTO.getMsgType(), KfAppletMsgDTO.MSG_TYPE_EVENT)
                && Objects.equals(kfAppletMsgDTO.getEvent(), KfAppletMsgDTO.EVENT_USER_ENTER_TEMP_SESSION)) {
            this.kfSessionUserUpdateSave(kfAppletMsgDTO, gameApplet);
            return;
        }
        //非玩家消息, 不做处理
        if (kfAppletMsgDTO.getMsgId() == null) {
            return;
        }
        //查询玩家的房间连接状态
        KfRoom kfRoom = kfRoomService.getOne(new LambdaQueryWrapper<KfRoom>()
                .eq(KfRoom::getOpenId, kfAppletMsgDTO.getFromUserName())
                .eq(KfRoom::getGameId, gameApplet.getGameId())
                .eq(KfRoom::getOnline, Boolean.TRUE));
        //消息存储
        KfWebSocketMsgDTO.MsgContentBean msgContent = this.getMsgContent(kfAppletMsgDTO);
        KfRoomMsg kfRoomMsg = this.transform(kfAppletMsgDTO, gameApplet, kfRoom, postData, msgContent);
        kfRoomMsgService.save(kfRoomMsg);
        //玩家状态更新为待接入状态
        if (kfRoom == null) {
            kfSessionUserService.update(new LambdaUpdateWrapper<KfSessionUser>()
                    .set(KfSessionUser::getIsWait, Boolean.TRUE)
                    .set(KfSessionUser::getWaitStartTime, LocalDateTime.now())
                    .set(KfSessionUser::getUpdateTime, LocalDateTime.now())
                    .eq(KfSessionUser::getOpenId, kfAppletMsgDTO.getFromUserName()));
        }
        //消息转发到redis频道
        this.pushMessage(this.transform(kfRoom, gameApplet.getGameId(), kfRoomMsg, msgContent));
    }

    private KfWebSocketMsgDTO transform(KfRoom kfRoom, Long gameId, KfRoomMsg kfRoomMsg, KfWebSocketMsgDTO.MsgContentBean msgContent) {
        //消息类型
        KfWebSocketMsgEnum kfWebSocketMsgEnum = kfRoom == null ? KfWebSocketMsgEnum.WEBSOCKET_MSG_WAIT_LIST
                : KfWebSocketMsgEnum.WEBSOCKET_MSG_ROOM_MSG;
        KfWebSocketMsgDTO kfWebSocketMsgDTO = KfWebSocketMsgDTO.builder()
                .webSocketMsgType(kfWebSocketMsgEnum)
                .kfUserId(kfRoom == null ? null : kfRoom.getKfUserId())
                .gameId(gameId)
                .roomId(kfRoom == null ? null : kfRoom.getId())
                .build();
        //待接入消息
        if (Objects.equals(kfWebSocketMsgEnum, KfWebSocketMsgEnum.WEBSOCKET_MSG_WAIT_LIST)) {
            List<KfWebSocketMsgDTO.WaitUserBean> waitUserList = kfSessionUserService.getWaitUserList(gameId);
            kfWebSocketMsgDTO.setWaitUserList(waitUserList);
        }
        //房间消息
        if (Objects.equals(kfWebSocketMsgEnum, KfWebSocketMsgEnum.WEBSOCKET_MSG_ROOM_MSG)) {
            //消息对象
            KfWebSocketMsgDTO.RoomMsgBean roomMsgBean = BeanUtil.copy(kfRoomMsg, KfWebSocketMsgDTO.RoomMsgBean.class);
            roomMsgBean.setContent(msgContent);
            kfWebSocketMsgDTO.setRoomMsgList(Collections.singletonList(roomMsgBean));
        }
        return kfWebSocketMsgDTO;
    }

    private void kfSessionUserUpdateSave(KfAppletMsgDTO kfAppletMsgDTO, GameApplet gameApplet) {
        KfSessionUser kfSessionUser = kfSessionUserService.getById(kfAppletMsgDTO.getFromUserName());
        //查询用户
        User user = userService.getOne(new LambdaQueryWrapper<User>().eq(User::getOpenId, kfAppletMsgDTO.getFromUserName()));
        //最近角色信息
        GameUserRole gameUserRole = null;
        if (user != null) {
            gameUserRole = gameUserRoleService.getLastGameUserRoleName(user.getId(), user.getGameId());
        }
        //不存在玩家信息, 创建保存
        if (kfSessionUser == null) {
            kfSessionUserService.save(this.transform(kfAppletMsgDTO, gameApplet, user, gameUserRole));
            return;
        }
        //存在, 更新玩家信息
        kfSessionUserService.update(new LambdaUpdateWrapper<KfSessionUser>()
                .set(KfSessionUser::getUserId, user == null ? null : user.getId())
                .set(KfSessionUser::getLastRoleId, gameUserRole == null ? "0" : gameUserRole.getRoleId())
                .set(KfSessionUser::getLastRoleName, gameUserRole == null ? kfAppletMsgDTO.getFromUserName() : gameUserRole.getRoleName())
                .set(KfSessionUser::getUpdateTime, LocalDateTime.now())
                .eq(KfSessionUser::getOpenId, kfAppletMsgDTO.getFromUserName()));
    }

    private KfSessionUser transform(KfAppletMsgDTO kfAppletMsgDTO, GameApplet gameApplet, User user, GameUserRole gameUserRole) {
        return KfSessionUser.builder()
                .openId(kfAppletMsgDTO.getFromUserName())
                .gameId(gameApplet.getGameId())
                .userId(user == null ? null : user.getId())
                .isWait(Boolean.FALSE)
                .lastRoleId(gameUserRole == null ? "0" : gameUserRole.getRoleId())
                .lastRoleName(gameUserRole == null ? kfAppletMsgDTO.getFromUserName() : gameUserRole.getRoleName())
                .createTime(LocalDateTime.now())
                .updateTime(LocalDateTime.now())
                .build();
    }

    private KfRoomMsg transform(KfAppletMsgDTO kfAppletMsgDTO, GameApplet gameApplet, KfRoom kfRoom, String postData,
                                KfWebSocketMsgDTO.MsgContentBean msgContent) {
        return KfRoomMsg.builder()
                .msgId(String.valueOf(kfAppletMsgDTO.getMsgId()))
                .msgType(kfAppletMsgDTO.getMsgType())
                .gameId(gameApplet.getGameId())
                .openId(kfAppletMsgDTO.getFromUserName())
                .readStatus(Boolean.FALSE)
                .roomId(kfRoom == null ? null : kfRoom.getId())
                .kfUserId(kfRoom == null ? null : kfRoom.getKfUserId())
                .msgOwner(KfRoomMsgOwnerEnum.KF_MSG_OWNER_USER.getValue())
                .content(JsonUtil.toString(msgContent))
                .source(postData)
                .createTime(LocalDateTime.now())
                .updateTime(LocalDateTime.now())
                .build();
    }

    private KfWebSocketMsgDTO.MsgContentBean getMsgContent(KfAppletMsgDTO kfAppletMsgDTO) {
        //todo : 这里要考虑图片是否要传到oss上, 用自己的网络地址, 腾讯的图片地址不能保证一直有效
        KfWebSocketMsgDTO.MsgContentBean msgContentBean = new KfWebSocketMsgDTO.MsgContentBean();
        if (Objects.equals(KfRoomMsgTypeEnum.KF_MSG_TYPE_TEXT.getValue(), kfAppletMsgDTO.getMsgType())) {
            msgContentBean.setText(kfAppletMsgDTO.getContent());
        }
        if (Objects.equals(KfRoomMsgTypeEnum.KF_MSG_TYPE_IMAGE.getValue(), kfAppletMsgDTO.getMsgType())) {
            msgContentBean.setImage(kfAppletMsgDTO.getPicUrl());
        }
        return msgContentBean;
    }

    /**
     * 消息发送到redis广播
     */
    private void pushMessage(KfWebSocketMsgDTO kfWebSocketMsgDTO) {
        redisTemplate.convertAndSend(RedisKeyConstant.KF_MSG_REDIS_LISTEN_TOPIC, JsonUtil.toString(kfWebSocketMsgDTO));
    }
}
