package com.zanxiang.game.module.manage.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.game.module.manage.enums.KfRoomMsgTypeEnum;
import com.zanxiang.game.module.manage.pojo.dto.KfWebSocketMsgDTO;
import com.zanxiang.game.module.manage.pojo.params.KfWebSocketMsgParam;
import com.zanxiang.game.module.manage.service.IKfRoomMsgService;
import com.zanxiang.game.module.mybatis.entity.KfRoomMsg;
import com.zanxiang.game.module.mybatis.mapper.KfRoomMsgMapper;
import com.zanxiang.module.util.JsonUtil;
import com.zanxiang.module.util.bean.BeanUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.stereotype.Service;
import reactor.util.function.Tuple2;
import reactor.util.function.Tuples;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @author : lingfeng
 * @time : 2024-02-23
 * @description : 客服聊天房间消息
 */
@Slf4j
@Service
public class KfRoomMsgServiceImpl extends ServiceImpl<KfRoomMsgMapper, KfRoomMsg> implements IKfRoomMsgService {

    @Override
    public Tuple2<KfWebSocketMsgDTO.PageBean, List<KfWebSocketMsgDTO.RoomMsgBean>> msgRoomHistory(Long roomId, KfWebSocketMsgParam.PageBean pageBean) {
        Page<KfRoomMsg> kfRoomMsgPage = super.page(new Page<>(pageBean.getPageNum(), pageBean.getPageSize()),
                new QueryWrapper<KfRoomMsg>().lambda()
                        .eq(KfRoomMsg::getRoomId, roomId)
                        .orderByDesc(KfRoomMsg::getCreateTime)
        );
        //构造消息列表
        List<KfWebSocketMsgDTO.RoomMsgBean> roomMsgBeanList = kfRoomMsgPage.getRecords().stream()
                .map(this::transform).collect(Collectors.toList());
        //更新消息已读状态
        List<String> msgIdList = roomMsgBeanList.stream()
                .filter(msg -> Objects.equals(msg.getReadStatus(), Boolean.FALSE))
                .map(KfWebSocketMsgDTO.RoomMsgBean::getMsgId)
                .collect(Collectors.toList());
        if (CollectionUtils.isNotEmpty(msgIdList)) {
            super.update(new LambdaUpdateWrapper<KfRoomMsg>()
                    .set(KfRoomMsg::getReadStatus, Boolean.TRUE)
                    .set(KfRoomMsg::getUpdateTime, LocalDateTime.now())
                    .in(KfRoomMsg::getMsgId, msgIdList));
        }
        //返回分页数据, 消息列表
        return Tuples.of(this.transform(kfRoomMsgPage), roomMsgBeanList);
    }

    private KfWebSocketMsgDTO.RoomMsgBean transform(KfRoomMsg kfRoomMsg) {
        if (kfRoomMsg == null) {
            return null;
        }
        KfWebSocketMsgDTO.RoomMsgBean roomMsgBean = BeanUtil.copy(kfRoomMsg, KfWebSocketMsgDTO.RoomMsgBean.class);
        roomMsgBean.setMsgType(KfRoomMsgTypeEnum.getMsgTypeEnum(kfRoomMsg.getMsgType()));
        roomMsgBean.setContent(JsonUtil.toObj(kfRoomMsg.getContent(), KfWebSocketMsgDTO.MsgContentBean.class));
        return roomMsgBean;
    }

    private KfWebSocketMsgDTO.PageBean transform(Page<KfRoomMsg> kfRoomMsgPage) {
        return KfWebSocketMsgDTO.PageBean.builder()
                .pageNum(kfRoomMsgPage.getCurrent())
                .pageSize(kfRoomMsgPage.getSize())
                .pageTotal(kfRoomMsgPage.getPages())
                .total(kfRoomMsgPage.getTotal())
                .build();
    }
}
