package com.zanxiang.game.module.manage.websocket;

import com.zanxiang.game.module.manage.pojo.dto.KfWebSocketMsgDTO;
import com.zanxiang.module.util.JsonUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.connection.Message;
import org.springframework.data.redis.connection.MessageListener;
import org.springframework.stereotype.Component;
import org.springframework.web.socket.TextMessage;
import org.springframework.web.socket.WebSocketSession;

import java.io.IOException;
import java.util.List;

/**
 * @author : lingfeng
 * @time : 2024-02-23
 * @description : 客服消息redis监听器
 */
@Component
@Slf4j
public class KfMsgRedisListener implements MessageListener {

    @Autowired
    private KfMsgWebSocketSessionRegistry kfMsgWebSocketSessionRegistry;

    @Override
    public void onMessage(Message message, byte[] pattern) {
        //从redis中拿到的消息
        String messageBody = new String(message.getBody());
        log.error("redis监听器监听到消息 messageBody : {}", messageBody);
        //转化消息对象
        KfWebSocketMsgDTO kfWebSocketMsgDTO = JsonUtil.toObj(messageBody, KfWebSocketMsgDTO.class);
        Long kfUserId = kfWebSocketMsgDTO.getKfUserId();
        //发送给指定客服
        if (kfUserId != null) {
            log.error("发送消息给指定客服 kfUserId : {}, kfWebSocketMsgDTO : {}", kfUserId, JsonUtil.toString(kfWebSocketMsgDTO));
            WebSocketSession session = kfMsgWebSocketSessionRegistry.getSession(kfUserId);
            if (session != null && session.isOpen()) {
                try {
                    session.sendMessage(new TextMessage(JsonUtil.toString(kfWebSocketMsgDTO)));
                } catch (Exception e) {
                    log.error("发送消息给指定客服异常, kfUserId : {}, kfWebSocketMsgDTO : {}, e : {}", kfUserId,
                            JsonUtil.toString(kfWebSocketMsgDTO), e.getMessage());
                }
            }
            return;
        }
        //发送给所有在线客服
        log.error("发送消息给所有客服 kfWebSocketMsgDTO : {}", JsonUtil.toString(kfWebSocketMsgDTO));
        List<WebSocketSession> openSessions = kfMsgWebSocketSessionRegistry.getAllSessions();
        openSessions.forEach(session -> {
            if (session != null && session.isOpen()) {
                try {
                    session.sendMessage(new TextMessage(JsonUtil.toString(kfWebSocketMsgDTO)));
                } catch (Exception e) {
                    log.error("发送消息给所有客服异常, kfUserId : {}, kfWebSocketMsgDTO : {}, e : {}",
                            session.getAttributes().get("kfUserId"), JsonUtil.toString(kfWebSocketMsgDTO), e.getMessage());
                }
            }
        });
    }
}
